import contextlib
import logging


logger = logging.getLogger(__name__)


class Stash(object):

    FILENAME = 'stash.db'

    @contextlib.contextmanager
    def open(self):
        import shelve
        with contextlib.closing(shelve.open(self.FILENAME)) as db:
            yield db

    def read(self):
        with self.open() as db:
            return dict(db)

    def write(self, data):
        with self.open() as db:
            for key, value in data.items():
                db[key] = value


class LazyStash(Stash):
    def __init__(self):
        self._stash = None

    @property
    def stash(self):
        if self._stash is None:
            self._stash = self.read()
        return self._stash

    def __getitem__(self, key):
        return self.stash[key]



from kibitzr.transformer.jinja_transform import JinjaTransform

from ...helpers import stash_mock


def jinja_transform(code, content, conf=None):
    return JinjaTransform(code, conf or {})(content)


def test_content_is_passed():
    ok, content = jinja_transform('hello {{ content }}!', 'world')
    assert ok is True
    assert content == "hello world!"


def test_lines_are_passed():
    ok, content = jinja_transform('{{ lines[1] }}', 'a\nb')
    assert ok is True
    assert content == "b"


def test_json_is_passed():
    ok, content = jinja_transform('{{ json["a"][1] }}', '{"a": [1, 2, 3]}')
    assert ok is True
    assert content == "2"


def test_css_selector_is_passed():
    ok, content = jinja_transform(
        '{{ css("div p") | text }}',
        '<div><a>A</a><p>P</p></div>',
    )
    assert ok is True
    assert content == "P"


def test_xpath_selector_is_passed():
    ok, content = jinja_transform(
        '{{ xpath("//div/p") | text }}',
        '<div><a>A</a><p>P</p></div>',
    )
    assert ok is True
    assert content == "P"


def test_stash_is_passed():
    with stash_mock() as stash:
        stash.write({'key': 'good'})
        ok, content = jinja_transform('{{ stash.key }} news', '')
        assert ok is True
        assert content == 'good news'
